/********************************************************************************** 
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <kapp.h>
#include <qpalette.h>
#include <qpainter.h>


#include "cube.h"
#include "cubesim.h"

// initialize static classelements

QPalette* Cube::color1;
QPalette* Cube::color2;
bool Cube::clicksAllowed=TRUE;
int Cube::playerPoints[2]={0,0};

// construktor of Cube

Cube::Cube(QWidget *parent,int row, int column , const char *name ,int max_Value) 
		:QFrame(parent,name)
{
  setFrameStyle(Panel|Raised);
  setLineWidth(2);
  this->row=row;
  this->column=column;
  points=1; // value of the cube
  owner=0; // set owner to nobody
  maxValue=max_Value;
  repaint();  // show value

}

Cube::~Cube()
{
  
}

// public functions

void Cube::setMaxValue(int wert)
{
  maxValue=wert;
}


void Cube::setColors(QPalette *player1, QPalette *player2)
{
  color1=player1;
  color2=player2;
}


void Cube::allowClicks(bool flag)
{
   clicksAllowed=flag;
}

// setOwner checks, if owner changed

void Cube::setOwner(int newOwner)
{ 
   if(newOwner==0 && owner!=0)   // wird bei undo bentigt
   {
      setPalette(*kapp->palette());
      playerPoints[owner-1]--;
      owner=0;
   }
   
  else if(newOwner==1 && owner==2)
    {
      setPalette(*color1); // backgroundcolor to color of player1
      owner=1; // set new owner
      playerPoints[1]--;
      playerPoints[0]++;
    }

  else if (newOwner==2 && owner==1)
    {
      setPalette(*color2); // backgroundcolor to color of player 2
      owner=2; // set new owner
      playerPoints[0]--;
      playerPoints[1]++;
    }  
  else if(owner==0)
    {
      if(newOwner==1)
	{ 
	  // backgroundcolor to color of player1
	  setPalette(*color1);
	  owner=1; // set new owner
	  playerPoints[0]++;
	}
      else if(newOwner==2)
	{// backgroundcolor to color of player 2
	  setPalette(*color2);
	  owner=2; // set new owner
	  playerPoints[1]++;
	}
    }

}

void Cube::setValue(int value)
{
  if(value > maxValue || value < 1)
    return;

  points=value;
  repaint();
}

int Cube::getOwner()
{
  return owner;
}

int Cube::getMaxValue()
{
  return maxValue;
}

int Cube::value()
{
  return points;
}



void Cube::increase(int player)
{ 
   setOwner(player);
    
    points++;
    
    if(points <= 9)  
      repaint();
    
}

void Cube::decrease()
{
  points=points-maxValue;

  if(points<9)
    repaint();
} 


int Cube::maxReached()
{
   if(points>maxValue)
      return 1;
   
   return 0;
}


void Cube::reset()
{ 
  points=1;
  owner=0;
  repaint();
  setPalette(*kapp->palette());
}

void Cube::updateColor()
{
  if(owner==1)
    setPalette(*color1);
  else if(owner==2)
    setPalette(*color2);
  else if(owner==0)
     setPalette(*kapp->palette());
}


void Cube::showTip()
{
   setPalette(palette().normal().light());
}

void Cube::hideTip()
{
   updateColor();
}



Cube& Cube::operator=(CubeSim c)
{
   setOwner(c.getOwner());
   setValue(c.getValue());
   
   return *this;
}


// EVENTS

void Cube::mouseReleaseEvent(QMouseEvent *e)
{
  // only accept click if it was inside this cube
  if(e->x()< 0 || e->x() > this->width() || e->y() < 0 || e->y() > this->height())
    return;

  if(e->button() == LeftButton && clicksAllowed)
    emit cubeClicked(row,column);
}


void Cube::drawContents(QPainter *p)
{
  int size=height();
  int circleSize=size/7;
  QBrush brush("black");
  QPen pen("black");
  p->setBrush(brush);
  p->setPen(pen);
  switch(points)
    {
    case 1:
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,circleSize,circleSize);
      break;

    case 3:
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,circleSize,circleSize);
    case 2:
      p->drawEllipse(size/4-circleSize/2,size/4-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/4-circleSize/2,
		     circleSize,circleSize);
      break;
      
    case 5:
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,circleSize,circleSize);
    case 4:
      p->drawEllipse(size/4-circleSize/2,size/4-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,size/4-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,3*size/4-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/4-circleSize/2,
		     circleSize,circleSize);
      break;

    case 8:
      p->drawEllipse(size/2-circleSize/2,2*size/3-circleSize/2,
		     circleSize,circleSize);
    case 7:
      p->drawEllipse(size/2-circleSize/2,size/3-circleSize/2,
		     circleSize,circleSize);
    case 6:
      p->drawEllipse(size/4-circleSize/2,size/6-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      break;
     
   
    case 9:
      p->drawEllipse(size/4-circleSize/2,size/6-circleSize/2,circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,3*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(3*size/4-circleSize/2,5*size/6-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/2-circleSize/2,2*size/7-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/2-circleSize/2,5*size/7-circleSize/2,
		     circleSize,circleSize);
      p->drawEllipse(size/2-circleSize/2,size/2-circleSize/2,
		     circleSize,circleSize);
      break; 

    default:
      debug("cube had value %d",points);
      QString s;
      s.sprintf("%d",points);
      p->drawText(size/2,size/2,s);
      break;
    }

}



//#include "cube.moc"










